/***************************************************************************
 *   Copyright (C) 2005-2007 Stefan Schwarzer, Jens Schneider,             *
 *                           Matthias Hardt, Guido Madaus                  *
 *                                                                         *
 *   Copyright (C) 2007-2008 Berlinux Solutions GbR                        *
 *                           Stefan Schwarzer & Guido Madaus               *
 *                                                                         *
 *   Authors:                                                              *
 *      Stefan Schwarzer <SSchwarzer@berlinux-solutions.de>,               *
 *      Matthias Hardt   <MHardt@berlinux-solutions.de>,                   *
 *      Jens Schneider   <pupeider@gmx.de>                                 *
 *      Guido Madaus     <GMadaus@berlinux-solutions.de>                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MMSWINDOWMANAGER_H_
#define MMSWINDOWMANAGER_H_

#include "mmsgui/interfaces/immswindowmanager.h"

class MMSWindowManager : public IMMSWindowManager {
	private:
		//! visible screen area
        MMSFBRectangle		vrect;

        //! windows known by the window manager
        vector<MMSWindow*>	windows;

        //! the toplevel window
        MMSWindow           *toplevel;

        //! the background window
        MMSWindow           *backgroundwindow;

        //! translator instance which can be used to translate text
        MMSTranslator		translator;

        //! connection object for onTargetLangChanged callback
        sigc::connection 	onTargetLangChanged_connection;


        void showBackgroundWindow();

        void onTargetLangChanged(MMS_LANGUAGE_TYPE lang);

	public:
		MMSWindowManager(MMSFBRectangle vrect);
		virtual ~MMSWindowManager();
		void reset();

        MMSFBRectangle getVRect();

		void addWindow(MMSWindow *window);
        void removeWindow(MMSWindow *window);

        bool lowerToBottom(MMSWindow *window);
		bool raiseToTop(MMSWindow *window);

        bool hideAllMainWindows(bool goback = false);
        bool hideAllPopupWindows(bool except_modal = false);
        bool hideAllRootWindows(bool willshown = false);

        bool setToplevelWindow(MMSWindow *window);
        MMSWindow *getToplevelWindow();
		void removeWindowFromToplevel(MMSWindow *window);

        void setBackgroundWindow(MMSWindow *window);
        MMSWindow *getBackgroundWindow();

        void setPointerPosition(int pointer_posx, int pointer_posy, bool pressed = true);

        MMSTranslator *getTranslator();
};

/* access to global mmswindowmanager */
//extern MMSWindowManager mmswindowmanager;

#endif /*MMSWINDOWMANAGER_H_*/
